/*
 * Created on Jan 14, 2005
 *
 */
package org.jboss.mail.store;

import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.system.ServiceMBean;
import org.jboss.system.ServiceMBeanSupport;
import org.jboss.util.id.GUID;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Abstact Store MBean contains all common functionality for the 
 * actual Store items.  Store's should inherit from this object.
 * 
 * @author Michael Barker <mailto:mike@middlesoft.co.uk>
 *
 */
public abstract class AbstractStore extends ServiceMBeanSupport implements ServiceMBean, Store
{
   private static Log log = LogFactory.getLog(AbstractStore.class); 
   private boolean compress = false;
   private int compressBufferSize = 8192;
   private int minBufferSize = 1024;
   private int maxBufferSize = 1024 * 1024;
   private int bufferSize = 8192;
   private boolean started;
   
   
   //==================
   // Store Methods
   //==================
   
   
   protected Object createId()
   {
      return GUID.asString();      
   }
   
   
   /* (non-Javadoc)
    * @see org.jboss.mail.store.Store#read(java.lang.Object, java.io.InputStream)
    */
   public void read(Object id, InputStream in)
   {
      // TODO Auto-generated method stub
      throw new UnsupportedOperationException("FIXME: Implement");
   }
   
   
   //=====================
   // Properties
   //=====================
   
   public void setCompress(boolean b)
   {
      this.compress = b;
   }
   public boolean isCompress()
   {
      return this.compress;
   }   
   /**
    * @return Returns the bufferSize.
    */
   public int getBufferSize()
   {
      return bufferSize;
   }
   /**
    * @param bufferSize The bufferSize to set.
    */
   public void setBufferSize(int bufferSize)
   {
      this.bufferSize = bufferSize;
   }
   /**
    * @return Returns the compressBufferSize.
    */
   public int getCompressBufferSize()
   {
      return compressBufferSize;
   }
   /**
    * @param compressBufferSize The compressBufferSize to set.
    */
   public void setCompressBufferSize(int compressBufferSize)
   {
      this.compressBufferSize = compressBufferSize;
   }
   
   /**
    * @return Returns the maxBufferSize.
    */
   public int getMaxBufferSize()
   {
      return maxBufferSize;
   }
   /**
    * @param maxBufferSize The maxBufferSize to set.
    */
   public void setMaxBufferSize(int maxBufferSize)
   {
      this.maxBufferSize = maxBufferSize;
   }
   /**
    * @return Returns the minBufferSize.
    */
   public int getMinBufferSize()
   {
      return minBufferSize;
   }
   /**
    * @param minBufferSize The minBufferSize to set.
    */
   public void setMinBufferSize(int minBufferSize)
   {
      this.minBufferSize = minBufferSize;
   }
   
   
   //===================
   // MBean Methods.
   //===================


   /* (non-Javadoc)
    * @see org.jboss.system.Service#start()
    */
   public void startService() throws Exception
   {
      validate();
      init();
      started = true;
   }
   
   /**
    * Validates the generic configuration properties.
    * Subclass should override this method and call
    * <code>super.validate()</code>.
    * 
    * @throws StoreException
    */
   public void validate() throws StoreException
   {
      
   }
   
   public abstract void init() throws StoreException;

   /* (non-Javadoc)
    * @see org.jboss.system.Service#stop()
    */
   public void stopService()
   {
      started = false;
   }

   /**
    * Configure the properties for this bean.
    * 
    * @param properties
    */
   public void setProperties(Element properties)
   {
      try
      {
         log.debug("setProperties()");
         NodeList nodes = properties.getElementsByTagName("property");
         for (int k = 0; k < nodes.getLength(); k++)
         {
             String pname = null;
             Object pval = null;
             Object[] pvals = new String[0];
             Node node = nodes.item(k);
             NodeList subnodes = node.getChildNodes();
             for (int x = 0; x < subnodes.getLength(); x++)
             {
                 Node subnode = subnodes.item(x);
                 if (subnode.getNodeName().equals("name")) 
                 {
                     pname = subnode.getFirstChild().getNodeValue();
                 }
                 else if (subnode.getNodeName().equals("value"))
                 {
                     pval = subnode.getFirstChild().getNodeValue();
                 }
                 else if (subnode.getNodeName().equals("values"))
                 {
                     NodeList values = subnode.getChildNodes();
                     List l = new ArrayList(values.getLength());
                     for (int y = 0; y < values.getLength(); y++)
                     {
                         Node value = values.item(y);
                         if (value.getNodeName() != null && value.getNodeName().equals("value")) 
                         {
                             l.add(value.getFirstChild().getNodeValue());
                         }
                     }
                     pvals = l.toArray(pvals);
                 }
             }
             if (pname != null && pval != null)
             {
                setProperty(pname, pval);
             }
             else if (pname != null && pvals.length > 0)
             {
                setProperty(pname, pvals);
             }
             else
             {
                log.warn("No property name or value specified name = " + pname + " value: " + pval);
             }
         }         
      }
      catch (Exception e)
      {
         log.error("Failed to configure bean: " + e.getMessage());
         throw new RuntimeException("Failed to configure bean: ", e);
      }
   }
   
   public void setProperty(String name, Object value) throws IllegalAccessException, InvocationTargetException
   {
      BeanUtils.setProperty(this, name, value);
   }
   
   public Object getProperty(String name) throws IllegalAccessException, InvocationTargetException, NoSuchMethodException
   {
      return BeanUtils.getProperty(this, name);
   }
   
   protected void testNull(String property) throws StoreException
   {
      try
      {
         if (getProperty(property) == null)
         {
            throw new StoreException(property + " has not been set");
         }
      }
      catch (Exception e)
      {
         throw new StoreException(e);
      }
   }

   
}
