/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.store;

import java.io.InputStream;
import java.io.OutputStream;
import java.nio.ByteBuffer;


/**
 * General interface for underlying storage.
 * 
 * @author Michael Barker <mailto:mike@middlesoft.co.uk>
 *
 */
public interface Store
{   
   
   public int read(byte[] b, int off, int len, Object id, long position) throws StoreException;

   public int write(byte[] b, int off, int len, Object id, long position) throws StoreException;  
      
   /**
    * Reads a ByteBuffer from the blob.  This is primarly use only for the blob Input/Output
    * Streams.
    * 
    * @param id
    * @param position
    * @param len
    * @return
    * @throws StoreException
    */
   public ByteBuffer getBuffer(Object id, long position, int len) throws StoreException;
   
   
   /**
    * Creates a store and will autogenerate and return the id.
    * @return
    * @throws StoreException
    */
   public StoreItem createStoreItem() throws StoreException;
   
   
   /**
    * Gets an input stream for the store
    * @throws StoreException 
    */
   public InputStream getInputStream(Object id) throws StoreException;
   
   
   
   public OutputStream getOuputStream(Object id) throws StoreException;

   /**
    * Read an entire input stream into this id.
    * 
    * @param id 
    * 
    * @param in
    */
   public void read(Object id, InputStream in);
   
   /**
    * Gets the meta data for the store item identified by id.
    * @param id
    * @return
    * @throws StoreException 
    */
   public StoreItemMetaData getStoreItemMetaData(Object id) throws StoreException;
}
