/*
 * Created on Jan 22, 2005
 *
 */
package org.jboss.mail.store.jdbc3;

import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;

import org.apache.commons.dbutils.DbUtils;

/**
 * Wraps up an InputStream from a blob with its associated connection
 * to allow for the connnection to be closed when the stream is.
 * 
 * @author Michael Barker <mailto:mike@middlesoft.co.uk>
 *
 */
public class WrappedInputStream extends InputStream
{
   private InputStream in;
   private Connection cn;

   public WrappedInputStream(InputStream in, Connection cn)
   {
      this.in = in;
      this.cn = cn;
   }

   /* (non-Javadoc)
    * @see java.io.InputStream#available()
    */
   public int available() throws IOException
   {
      return in.available();
   }
   /* (non-Javadoc)
    * @see java.io.InputStream#close()
    */
   public void close() throws IOException
   {
      try
      {
         in.close();         
      }
      finally
      {
         DbUtils.closeQuietly(cn);
      }
   }
   /* (non-Javadoc)
    * @see java.lang.Object#equals(java.lang.Object)
    */
   public boolean equals(Object arg0)
   {
      return in.equals(arg0);
   }
   /* (non-Javadoc)
    * @see java.lang.Object#hashCode()
    */
   public int hashCode()
   {
      return in.hashCode();
   }
   /* (non-Javadoc)
    * @see java.io.InputStream#mark(int)
    */
   public void mark(int arg0)
   {
      in.mark(arg0);
   }
   /* (non-Javadoc)
    * @see java.io.InputStream#markSupported()
    */
   public boolean markSupported()
   {
      return in.markSupported();
   }
   /**
    * @return
    * @throws java.io.IOException
    */
   public int read() throws IOException
   {
      return in.read();
   }
   /* (non-Javadoc)
    * @see java.io.InputStream#read(byte[])
    */
   public int read(byte[] arg0) throws IOException
   {
      return in.read(arg0);
   }
   /* (non-Javadoc)
    * @see java.io.InputStream#read(byte[], int, int)
    */
   public int read(byte[] arg0, int arg1, int arg2) throws IOException
   {
      return in.read(arg0, arg1, arg2);
   }
   /* (non-Javadoc)
    * @see java.io.InputStream#reset()
    */
   public void reset() throws IOException
   {
      in.reset();
   }
   /* (non-Javadoc)
    * @see java.io.InputStream#skip(long)
    */
   public long skip(long arg0) throws IOException
   {
      return in.skip(arg0);
   }
   /* (non-Javadoc)
    * @see java.lang.Object#toString()
    */
   public String toString()
   {
      return in.toString();
   }
}
