/*
 * Created on Jan 3, 2005
 *
 */
package org.jboss.mail.store;

import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import javax.sql.DataSource;

import org.apache.commons.dbutils.DbUtils;

/**
 * @author Michael Barker <mailto:mike@middlesoft.co.uk>
 *
 */
public class TestUtil
{
   public static DataSource getDataSource(String className, String url, String user, String pass)
   {
      return new MyDataSource(className, url, user, pass);
   }
   
   public static void printMemInfo()
   {
      long tm = Runtime.getRuntime().totalMemory();
      long mm = Runtime.getRuntime().maxMemory();
      long fm = Runtime.getRuntime().freeMemory();
      long afm = (mm - tm) + fm;
      System.out.println("Memory: " + afm/1000 + "K of " + mm/1000 + "K");
   }
   
   
   public static void execute(DataSource ds, String[] statements) throws SQLException
   {
      Connection con = null;
      Statement stmt = null;
      try
      {
         con = ds.getConnection();
         
         for (int i = 0; i < statements.length; i++)
         {
            stmt = con.createStatement();
            System.out.println("Executing: " + statements[i]);
            stmt.execute(statements[i]);
            stmt.close();
         }
         
      }
      finally
      {
         DbUtils.closeQuietly(stmt);
         DbUtils.closeQuietly(con);
      }
   }
   
   
   static class MyDataSource implements DataSource
   {
      private String className;
      private String url;
      private String user;
      private String pass;

      public MyDataSource(String className, String url, String user, String pass)
      {
         this.className = className;
         
         try
         {
            // Check to see if driver exists in the classpath.
            Class c = Class.forName(className);
         }
         catch (Exception e)
         {
            throw new RuntimeException("Unable to location driver: " + className);
         }
         
         this.url = url;
         this.user = user;
         this.pass = pass;
      }

      /* (non-Javadoc)
       * @see javax.sql.DataSource#getConnection()
       */
      public Connection getConnection() throws SQLException
      {
         try
         {
            Class c = Class.forName(className);
            DriverManager.registerDriver((Driver) c.newInstance());
            return DriverManager.getConnection(url, user, pass);
         }
         catch (SQLException e)
         {
            throw e;
         }
         catch (Exception e)
         {
            throw new RuntimeException(e);
         }
      }

      /* (non-Javadoc)
       * @see javax.sql.DataSource#getConnection(java.lang.String, java.lang.String)
       */
      public Connection getConnection(String username, String password) throws SQLException
      {
         try
         {
            Class c = Class.forName(className);
            DriverManager.registerDriver((Driver) c.newInstance());
            return DriverManager.getConnection(url, username, password);
         }
         catch (SQLException e)
         {
            throw e;
         }
         catch (Exception e)
         {
            throw new RuntimeException(e);
         }
      }

      /* (non-Javadoc)
       * @see javax.sql.DataSource#getLogWriter()
       */
      public PrintWriter getLogWriter() throws SQLException
      {
         // TODO Auto-generated method stub
         return null;
      }

      /* (non-Javadoc)
       * @see javax.sql.DataSource#setLogWriter(java.io.PrintWriter)
       */
      public void setLogWriter(PrintWriter arg0) throws SQLException
      {
         // TODO Auto-generated method stub
         
      }

      /* (non-Javadoc)
       * @see javax.sql.DataSource#setLoginTimeout(int)
       */
      public void setLoginTimeout(int arg0) throws SQLException
      {
         // TODO Auto-generated method stub
         
      }

      /* (non-Javadoc)
       * @see javax.sql.DataSource#getLoginTimeout()
       */
      public int getLoginTimeout() throws SQLException
      {
         // TODO Auto-generated method stub
         return 0;
      }
      
   }
   
}
